
import appuifw
import sysinfo
import e32
import contacts
from graphics import *
import struct
from math import cos, pi, fabs, log10
import audio
import os
import time
import telephone

# Graphics mode: -1 = deactivated, 1 = activated
draw_graphics = 1
agent_name = u""
selected_phone_number = u"0"
loudness_threshold = 70

# Screen size: large shows softkey labels, full is the whole screen
appuifw.app.screen='large'

# Global variables for calculating volume
samplerate = 0
dataLength = 0
f = 0
hamming = 0
isrecording = None
callState = telephone.EStatusIdle

# Generates a Hamming window that is needed for block processing.
def calcHammingWindow(n):
	global hamming
	
	k = 0
	while k < n:
		hamming[k] = 0.54 - 0.46 * cos(2*pi*(k/(n-1)))
		k = k + 1

# Reads the information stored in the header of the wave file.
# If printing is True, this information is printed to the console.
def readHeaderInfos(filename, printing):
	global samplerate
	global dataLength
	global f
	
	riff = f.read(4)
	fileLen = struct.unpack("i", f.read(4))[0]
	wave = f.read(4)
	fmt = f.read(4)
	fmtLen = struct.unpack("i", f.read(4))[0]
	compression = struct.unpack("h", f.read(2))[0]
	channels = struct.unpack("h", f.read(2))
	samplerate = struct.unpack("i", f.read(4))[0]
	bytesPerSec = struct.unpack("i", f.read(4))[0]
	bytesPerSample = struct.unpack("h", f.read(2))[0]
	bits = struct.unpack("h", f.read(2))[0]
	
	if printing == True:
		print u"Name: ", filename
		print u"RIFF-Chunk: ", riff
		print u"File length: %d" % fileLen
		print u"WAVE-Chunk: ", wave
		print u"FMT-Chunk:", fmt
		print u"FMT length: %d" % fmtLen
		print u"Compression: %d" % compression
		print u"Channels: %d" % channels
		print u"Samplerate: %d" % samplerate
		print u"Bytes per Second: %d" % bytesPerSec
		print u"Bytes per Sample: %d" % bytesPerSample
		print u"Bits per Sample: %d" % bits
	
	while True:
		tmp = f.read(1)
		if isinstance(tmp, str):
			tmp = tmp.upper()
			if tmp == "D":
				tmp2 = f.read(1)
				if isinstance(tmp2, str):
					tmpstr = tmp + tmp2
					tmpstr = tmpstr.upper()
					if tmpstr == u"DA":
						tmp2 = f.read(1)
						if isinstance(tmp2, str):
							tmpstr = tmpstr + tmp2
							tmpstr = tmpstr.upper()
							if tmpstr == u"DAT":
								tmp2 = f.read(1)
								if isinstance(tmp2, str):
									tmpstr = tmpstr + tmp2
									tmpstr = tmpstr.upper()
									if tmpstr == u"DATA":
										if printing == True:
											print u"DATA-Chunk: ", tmpstr
										break
	dataLength = struct.unpack("i", f.read(4))[0]
	if printing == True:
		print u"Data length: %d" % dataLength
		print u"End of Header Information"

# Calculates the Volume of the recorded sound file.
def	calcVolume(filename):
	global samplerate
	global dataLength
	global f
	global hamming
	global isrecording
	global loudness_threshold

	position = 0
	runsPerSecond = 3
	offset = 20
	nullValue = pow(0.0000000000000001, 2)

	try:
		
		f = open("e:\\python\\Spyphone\\" + filename, 'r+')
		
		readHeaderInfos(filename, 0)
		
		numOfSamples = dataLength/2
		blocksize = round(samplerate/8)
		step = round(samplerate/runsPerSecond)
		
		hamming = range(blocksize)
		calcHammingWindow(blocksize)
		
		numOfDbValues = round(numOfSamples / step)
		dB = range(numOfDbValues)
		
		i = 0
		while (i + blocksize) < numOfSamples:
			sum = 0
			idx = 0
			while idx < blocksize:
				val = fabs((((struct.unpack("h", f.read(2))[0]) * 2.0 )/ 65536) * hamming[idx])
				if val == 0:
					val = nullValue
				else:
					val = pow(val, 2)
				sum = sum + val
				idx = idx + 1
				
			f.read(2*(step-blocksize))
			signalEnergy = sum / blocksize
			dB[position] = 20 * log10(signalEnergy / 0.00002) + offset
			if dB[position] > loudness_threshold:
				appuifw.note(u"Threshold exceeded", "info")
				isrecording = None
				appuifw.app.menu = [(u"Start Observation", menu_calcVolume),(u"Agent from Contacts", menu_contacts),(u"Set Agent", menu_setAgent), (u"Set Threshold", menu_setThreshold), (u"About", menu_about),(u"Exit", menu_exit)]
				call_phoneNumber()
				break
			position = position + 1
			i = i + step

		# print result
		for j in dB:
			print j
			
		f.close()
	except IOError, reason:
		print reason
		print reason.filename
		print reason.errno
		print reason.strerror

# Is called when the application shall be closed.
def exit_key_handler():
    app_lock.signal()

# Displays information about the application.
def menu_about():
    appuifw.note(u"Daniel Haslinger and Clemens Rainer, 2008", "info")

# Is called when the application shall be closed by choosing the menu entry.
def menu_exit():
    exit_key_handler()

# Starts recording and calculating.
# File 1 is recorded while the volume of file 2 is calculated and the other way round.
def menu_calcVolume():
	global f
	global isrecording
	print "enter calcVolume"
	
	isrecording = appuifw.query(u"Please press OK to start observation.", "query")
	if isrecording == None:
		return;
	
	appuifw.app.menu = [(u"Stop Observation", menu_stopCalc),(u"Agent from Contacts", menu_contacts),(u"Set Agent", menu_setAgent), (u"Set Threshold", menu_setThreshold), (u"About", menu_about),(u"Exit", menu_exit)]

	filename1 = "test1.wav"
	filename2 = "test2.wav"
	
	# set recording time to 5 seconds
	recTime = 5
	firsttime = True
	
	while True:
		if isrecording == True:
			# open audio stream 1
			s1 = audio.Sound.open("e:\\python\\Spyphone\\" + filename1)
			# start recording s1
			s1.record()
			if firsttime == False:
				# analyze audio file 2
				calcVolume(filename2)
				# delete audio file 2
				try:	
					os.remove('e:\\python\\Spyphone\\' + filename2)
				except IOError, reason:
					print reason
					print reason.filename
					print reason.errno
					print reason.strerror
				# wait for recTime seconds
				e32.ao_sleep(recTime)
			else:
				# wait for recTime seconds
				e32.ao_sleep(recTime)
				# set recTime to 2 seconds
				recTime = 2
				# set firsttime false
				firsttime = False
			# stop recording s1
			s1.stop()
			# close s1
			s1.close()
		else:
			if firsttime == False:
				# delete audio file 2
				try:
					os.remove('e:\\python\\Spyphone\\' + filename2)
				except IOError, reason:
					print reason
					print reason.filename
					print reason.errno
					print reason.strerror
			break
		if isrecording == True:
			# open audio stream 2
			s2 = audio.Sound.open("e:\\python\\Spyphone\\" + filename2)
			# start recording s2
			s2.record()
			# analyze audio file 1
			calcVolume(filename1)
			# delete audio file 1
			try:	
				os.remove('e:\\python\\Spyphone\\' + filename1)
			except IOError, reason:
				print reason
				print reason.filename
				print reason.errno
				print reason.strerror
			# wait for recTime seconds
			e32.ao_sleep(recTime)
			# stop recording s2
			s2.stop()
			# close s2
			s2.close()
		else:
			# delete audio file 1
			try:	
				os.remove('e:\\python\\Spyphone\\' + filename1)
			except IOError, reason:
				print reason
				print reason.filename
				print reason.errno
				print reason.strerror
			break
	
	print "leave calcVolume"

# Stops the recording and the calculating.
def menu_stopCalc():
	global isrecording
	
	print "enter stopCalc"
	isrecording = None
	appuifw.app.menu = [(u"Start Observation", menu_calcVolume),(u"Agent from Contacts", menu_contacts),(u"Set Agent", menu_setAgent), (u"Set Threshold", menu_setThreshold), (u"About", menu_about),(u"Exit", menu_exit)]
	print "leave stopCalc"
	
# Displays the contact list.
# The user can choose the number to be called when the dB threshold is exceeded.
def menu_contacts():
	global selected_phone_number
	global agent_name
	global img
	
	db = contacts.open()
		
	all_ids = db.keys()
	
	found = db.find('')  # search in name, email, etc.

	print "BlaBlaBlaBla"
	print len(found)
	entries = []

	i = 0
	for id in found:
		cont = found[i]
		if cont != None:
			firstname = u""
			lastname = u""
			if len(cont.find('first_name')) > 0:
				#print cont.find('first_name')[0].value
				firstname = cont.find('first_name')[0].value
				firstname = firstname + " "
			if len(cont.find('last_name')) > 0:
				#print cont.find('last_name')[0].value
				lastname = cont.find('last_name')[0].value
			
			firstname = firstname + lastname
			entries.append(firstname)
			#entries.append(u"xxx")
		i = i+1

	entries.sort();
	idx = appuifw.popup_menu(entries, u"Contacts:")
	if idx > -1:
		firstname = entries[idx]
		blank_idx = firstname.find(" ")
		lastname = firstname[blank_idx+1:len(firstname)]
		firstname = firstname[0:blank_idx]
		found = db.find(lastname)
		cont = found[0]
		if len(found) > 1:
			i = 0
			for id in found:
				contact = found[i]
				f_name = contact.find('first_name')[0].value
				if f_name == firstname:
					cont = contact
				i = i+1
		tel_num = cont.find('mobile_number')[0].value
		selected_phone_number = tel_num
		agent_name = entries[idx]
		
		# Add menu item to start observation.
		appuifw.app.menu = [(u"Start Observation", menu_calcVolume),(u"Agent from Contacts", menu_contacts),(u"Set Agent", menu_setAgent), (u"Set Threshold", menu_setThreshold), (u"About", menu_about),(u"Exit", menu_exit)]
	
		# clear (reload) the image
		load_image()
		
		img.text((15, 225),u"Agent:",0x000000,font="normal")
		img.text((30, 250),u"%s"%entries[idx],0x000000,font="legend")
		img.text((30, 275),u"# %s"%tel_num,0x000000,font="dense")
	
		handle_redraw(())
		
		store_configs();

# User can type in name an number of the agent.
def menu_setAgent():
	global selected_phone_number
	global agent_name
	
	agentName = appuifw.query(u"Name:", "text")
	if agentName != None:
		number = appuifw.query(u"Number:", "text")
		
		if number != None:
			# Add menu item to start observation.
			appuifw.app.menu = [(u"Start Observation", menu_calcVolume),(u"Agent from Contacts", menu_contacts),(u"Set Agent", menu_setAgent), (u"Set Threshold", menu_setThreshold), (u"About", menu_about),(u"Exit", menu_exit)]

			agent_name = agentName
			selected_phone_number = number
			
			# clear (reload) the image
			load_image()
			
			img.text((15, 225),u"Agent:",0x000000,font="normal")
			img.text((30, 250),u"%s"%agent_name,0x000000,font="legend")
			img.text((30, 275),u"# %s"%selected_phone_number,0x000000,font="dense")

			handle_redraw(())
			
			store_configs();

# Sets the loudness threshold
def menu_setThreshold():
	global loudness_threshold
	
	loudness_threshold = appuifw.query(u"Set dB threshold for observation.", "number", loudness_threshold)
	store_configs();
	
# Reads the config data from the file.
def read_configs():
	global selected_phone_number
	global loudness_threshold
	global agent_name
	
	try:
		f = file(u"e:\\python\\Spyphone\\config.sp", "r")
		error = True
		
		line = f.readline()
		if cmp(line, u"#BEGIN\r\n") == 0:
			line = f.readline()
			if cmp(line, u"#Spyphone-Config\r\n") == 0:
				line = f.readline()
				if cmp(line, u"#Agent-Name\r\n") == 0:
					line = f.readline()
					agent_name = line[0:len(line)-2]
					line = f.readline()
					if cmp(line, u"#Number\r\n") == 0:
						line = f.readline()
						selected_phone_number = line[0:len(line)-2]
						line = f.readline()
						if cmp(line, u"#Loudness\r\n") == 0:
							line = f.readline()
							if not cmp(line, u"\r\n") == 0:
								loudness_threshold = int(line)
							line = f.readline()
							if cmp(line, u"#END\r\n") == 0:
								error = False
		if error:
			appuifw.note(u"Error while reading configs", "error")
		else:
			img.text((15, 225),u"Agent:",0x000000,font="normal")
			img.text((30, 250),u"%s"%agent_name,0x000000,font="legend")
			img.text((30, 275),u"# %s"%selected_phone_number,0x000000,font="dense")

			handle_redraw(())
			
		f.close()
	except IOError, reason:
		print reason
		print reason.filename
		print reason.errno
		print reason.strerror
	
# Stores the given agent data to a config file.
def store_configs():
	global selected_phone_number
	global agent_name
	global loudness_threshold
	
	try:
		os.remove(u"e:\\python\\Spyphone\\config.sp")
		f = file(u"e:\\python\\Spyphone\\config.sp", "w+")
		f.seek(0)
		f.write(u"#BEGIN\r\n")
		f.write(u"#Spyphone-Config\r\n")
		f.write(u"#Agent-Name\r\n")
		f.write(str(agent_name) + u"\r\n")
		f.write(u"#Number\r\n")
		f.write(str(selected_phone_number) + u"\r\n")
		f.write(u"#Loudness\r\n")
		f.write(str(loudness_threshold) + u"\r\n")
		f.write(u"#END\r\n")
		f.close()
	except IOError, reason:
		print reason
		print reason.filename
		print reason.errno
		print reason.strerror

	
# Calls the selected_phone_number.
# Is called when a volume value exceeds the threshold.
def call_phoneNumber():
	global selected_phone_number
	global callState
	
	observationTime = 15;
	telephone.call_state(call_func)

	no = str(selected_phone_number)
	if no[0]==u"0":
		number = u"+43" + no[1:len(no)]
		try:
			telephone.dial(number)
		except:
			appuifw.note(u"Trying to dial invalid phone number!", "error")
		print number
	elif no[0]==u"+":
		try:
			telephone.dial(no)
		except:
			appuifw.note(u"Trying to dial invalid phone number!", "error")
		print no
	else:
		appuifw.note(u"Number is invalid!", "error")
		return
		
	print u"number called"
	
def call_func(state):
	callState = state[0]
	print state[0]
	if str(callState) == u"5": # 5 = Status connected
		e32.ao_sleep(15)
		print u"hang up"
		telephone.hang_up()
	
def menu_stopCall():
	hang_up();
	
# Redraws the screen, in this case the image shall be
# drawn again and again (use the .blit function for this)
def handle_redraw(rect):
    global img
    canvas.blit(img)

# Function to load the background image
def load_image():
	global img
	try:
		img=Image.open(u"e:\\python\\Spyphone\\Logo.png")
	except IOError, reason:
		print reason
		print reason.filename
		print reason.errno
		print reason.strerror

# Initialize the application
appuifw.app.title = u"BabySpyPhone"
appuifw.app.exit_key_handler = exit_key_handler

# Check if the needed Spyphone directory is available.
if not os.path.isdir(u"e:\\python\\Spyphone"):
	appuifw.note(u"Directory missing! Application has to be closed!", "error")
else:
	# create an empty image (appears as white image)
	res_x, res_y = sysinfo.display_pixels()
	img = Image.new((res_x,res_y))

	load_image()
		
	#handle_redraw(())

	if draw_graphics == 1:
	    # Use the graphical visualization
	    canvas=appuifw.Canvas(redraw_callback=handle_redraw)
	    # set the application body as canvas
	    appuifw.app.body=canvas
	else:
	    # Just set the keyboard handler and don't draw anything so that
	    # print messages can be seen on the console
	    canvas=appuifw.Canvas()

	read_configs()
	
	if agent_name != None and agent_name != u"":
		appuifw.app.menu = [(u"Start Observation", menu_calcVolume),(u"Agent from Contacts", menu_contacts),(u"Set Agent", menu_setAgent), (u"Set Threshold", menu_setThreshold), (u"About", menu_about),(u"Exit", menu_exit)]
	else:
		appuifw.app.menu = [(u"Agent from Contacts", menu_contacts),(u"Set Agent", menu_setAgent), (u"Set Threshold", menu_setThreshold), (u"About", menu_about),(u"Exit", menu_exit)]
		
	app_lock = e32.Ao_lock()
	app_lock.wait()